#!/usr/bin/env python
# Copyright (C) 2022 Torbjorn Sjostrand.
# PYTHIA is licenced under the GNU GPL v2 or later, see COPYING for details.
# Please respect the MCnet Guidelines, see GUIDELINES for details.
# Author: Philip Ilten, October 2022.

# This is a script to check if the particle data is sensible.
# Run as "./private/particleData.py".

# Simple method to display a message if not empty.
def display(msg):
    for line in ["-"*80, msg[0], "-"*80] + msg[1:]: print(line)

# Load the ParticleDatabase class and create a database.
import sys
sys.path.insert(0, "examples")
from main39 import ParticleDatabase
pdb = ParticleDatabase()
for name, pde in pdb.items():
    if type(name) == int or pde.id < 0: del pdb[name]

#==========================================================================

msg = ["Neutral particle mismatch with name."]
for name, pde in pdb.items():
    if pde.isDiquark(): continue
    if pde.chargeType != 0 and pde.name.endswith("0"): msg += ["-"*7, str(pde)]
display(msg)
        
#==========================================================================

msg = ["Charged particle mismatch with name."]
for name, pde in pdb.items():
    if pde.isDiquark(): continue
    if pde.isQuark(): continue
    if pde.isNucleus(): continue
    if pde.n[2:5] == (0, 0, 0) and pde.n[0] + 10*pde.n[1] != 0: continue
    if pde.chargeType != 3*pde.name.count("+") - 3*pde.name.count("-"):
        msg += ["-"*7 + "\n" + str(pde)]
display(msg)
        
#==========================================================================

msg = ["Baryon particle/anti-particle name mismatch."]
for name, pde in pdb.items():
    if not pde.isBaryon(): continue
    if pde.chargeType == 0: name = "bar0".join(name.rsplit("0", 1))
    elif pde.chargeType > 0:
        name = name.replace("+", "bar-", 1).replace("+", "-")
    elif pde.chargeType < 0:
        name = name.replace("-", "bar+", 1).replace("-", "+")
    if pde.anti.name != name: msg += ["%20s %20s" % (name, pde.anti.name)]
display(msg)
    
#==========================================================================

msg = ["Meson particle/anti-particle name mismatch."]
for name, pde in pdb.items():
    if not pde.isMeson() or pde.anti == None: continue
    if pde.chargeType == 0: name = "bar0".join(name.rsplit("0", 1))
    elif pde.chargeType > 0: name = name.replace("+", "-")
    elif pde.chargeType < 0: name = name.replace("-", "+")
    if pde.anti.name != name: msg += ["%20s %20s" % (name, pde.anti.name)]
display(msg)

#==========================================================================

msg = ["Mismatch between lifetime and width."]
for name, pde in pdb.items():
    if pde.mWidth != 0 and pde.tau0 != float("inf"): msg += ["-"*6, str(pde)]
display(msg)

#==========================================================================

msg = ["Narrow width with no lifetime."]
hbar = 6.58211951e-25 # Reduced Planck's constant (GeV s).
for name, pde in pdb.items():
    if pde.mWidth == 0: continue
    if hbar/pde.mWidth > 1e-10: msg += ["-"*6, hbar/pde.mWidth, str(pde)]
display(msg)

#==========================================================================

msg = ["Nominal mass not within min/max range."]
for name, pde in pdb.items():
    if not pde.mMin <= pde.m0 <= pde.mMax: msg += ["-"*6, str(pde)]
display(msg)
